// --------------------------------------------------------------------------
// External files inclusions.
// --------------------------------------------------------------------------
#include <pic.h>                                                // Use a PIC16F877A.
#include "io.h"                                                 // I/O lines definitions.
#include "rs232.h"                                              // RS232 definitions.



// --------------------------------------------------------------------------
// RS232 constants and settings.
// --------------------------------------------------------------------------
const long int f_xtal = 18432000;                               // Crystal frequency.
const long int baud_rate = 9600;                                // Desired baud rate (check for overflow and verify rounding error).



// --------------------------------------------------------------------------
// This procedure will set up the serial port for transmitting and receiving
// data. The format is "baud_rate"N81. "f_xtal" must be set to generate the
// correct baud rate. No flow control is implemented.
// --------------------------------------------------------------------------
void RS232_init(void)
{
    SPBRG = (f_xtal / (16 * baud_rate)) - 1;                    // Set baud rate, high speed (BRGH=1)
//    SPBRG = (f_xtal / (64 * baud_rate)) - 1;                    // Set baud rate, low speed (BRGH=0)
    RCSTA = 0b10010000;
//            
//            +-- RX9D 9th bit of data (read only, not used).
//            +--- OERR Overrun error (read only).
//            +---- FERR Frame error (read only).
//            +-----      Unumplemented.
//            +------ CREN Continuous receive (enables reception).
//            +------- SREN Don't care.
//            +-------- RX9  Select 8 bit of data.
//            +--------- SPEN Serial port enabled.
    TXSTA = 0b00100100;
//            
//            +-- TX9D 9th bit of data (not used).
//            +--- TRMT TX status bit (read only).
//            +---- BRGH Select high speed mode (see SPBRG).
//            +-----      Unumplemented.
//            +------ SYNC Select asynchronous mode.
//            +------- TXEN Enable transmitter.
//            +-------- TX9  Select 8 bit of data.
//            +--------- CSRC Don't care.
    CREN = 1;                                                   // Make sure reception is enabled.
    RS232_CTS = 0;                                              // Set CTS (inverted) to allow PC using RS232 port (even if flow control is not implemented).
}



// --------------------------------------------------------------------------
// This procedure sends the specified byte over the RS232 port.
// --------------------------------------------------------------------------
void RS232_send_byte(unsigned char data)
{
    while (TRMT == 0);                                          // Wait until transmit buffer is empty.
    TXREG = data;                                               // Transmit data.
}



// --------------------------------------------------------------------------
// This procedure sends the specified null terminated string over the serial
// port.
// --------------------------------------------------------------------------
void RS232_send_string(const unsigned char * string)
{
    unsigned char char_pointer;                                 // Pointer at the current character.
    
    char_pointer = 0;                                           // Start with first character.
    while (string[char_pointer] != 0)                           // Parse the string untill a null is found.
    {
        RS232_send_byte(string[char_pointer]);                  // Transmit character.
        char_pointer++;
    }
}



// --------------------------------------------------------------------------
// This procedure checks if a byte was received and returns it. If nothing
// was received RS232_NONE (-1) is returned and if an error is detected
// RS232_ERR (-2) is returned.
// --------------------------------------------------------------------------
signed int RS232_receive_data(void)
{
    if (RCIF == 0)                                              // Check if something was received.
        return(RS232_NONE);                                     // No.
    if (OERR == 1)                                              // Check for overrun error (framing errors do not require reset and returns real data)
    {
        CREN = 0;                                               // Reset receiver.
        NOP();
        CREN = 1;
        return(RS232_ERR);                                      // Return error.
    }
    return(RCREG);                                              // Return received data.
}



// --------------------------------------------------------------------------
// This procedure resets the RS232 receiver.
// --------------------------------------------------------------------------
void RS232_receiver_reset(void)
{
    CREN = 0;                                                   // Reset receiver.
    NOP();
    CREN = 1;
}


